
#include <avr/io.h>
#include <avr/interrupt.h>
#include <stdlib.h>
#include "uart.h"
#include "lcd_new.h"

#define RC5TIME 	1.778e-3		// 1.778msec
#define PULSE_MIN	(uchar)(F_CPU / 512 * RC5TIME * 0.4 + 0.5)
#define PULSE_1_2	(uchar)(F_CPU / 512 * RC5TIME * 0.8 + 0.5)
#define PULSE_MAX	(uchar)(F_CPU / 512 * RC5TIME * 1.2 + 0.5)
#define uchar unsigned char
#define uint unsigned int
#define UART_BAUD_RATE     9600 // Bautrate UART 
#define	xRC5_IN		PIND
#define	xRC5		PD2			// IR input low active

uchar	rc5_bit;				// bit value
uchar	rc5_time;				// count bit time
uint	rc5_tmp;				// shift bits in
volatile uint	rc5_data;				// store result


ISR(TIMER0_OVF_vect)
{
  uint tmp = rc5_tmp;				// for faster access

  TCNT0 = -2;					// 2 * 256 = 512 cycle

  if( ++rc5_time > PULSE_MAX ){			// count pulse time
    if( !(tmp & 0x4000) && tmp & 0x2000 )	// only if 14 bits received
      rc5_data = tmp;
    tmp = 0;
  }

  if( (rc5_bit ^ xRC5_IN) & 1<<xRC5 ){		// change detect
    rc5_bit = ~rc5_bit;				// 0x00 -> 0xFF -> 0x00

    if( rc5_time < PULSE_MIN )			// to short
      tmp = 0;

    if( !tmp || rc5_time > PULSE_1_2 ){		// start or long pulse time
      if( !(tmp & 0x4000) )			// not to many bits
        tmp <<= 1;				// shift
      if( !(rc5_bit & 1<<xRC5) )		// inverted bit
        tmp |= 1;				// insert new bit
      rc5_time = 0;				// count next pulse time
    }
  }

  rc5_tmp = tmp;
}

int main( void )
{
  
  DDRA=0xff;
  uint i;
  char s[30];

  TCCR0 = (1<<CS02);			//divide by 64
  TIMSK = 1<<TOIE0;			//enable timer interrupt

uart_init(UART_BAUD_SELECT(UART_BAUD_RATE,F_CPU));  //UART parametrieren
lcd_init(LCD_DISP_ON);

  sei();
lcd_puts("RC5 Test"); 
uart_puts("RC5 Test\n\r");

  for(;;){				// main loop
    cli();
    i = rc5_data;			// read two bytes from interrupt !
    rc5_data = 0;
    sei();
    if( i ){
      PORTA = i;				// LED output
      uart_putc(( i >> 11 & 1) + '0');	// Toggle Bit
      uart_putc(' ');
	  lcd_putc(( i >> 11 & 1) + '0');	// Toggle Bit
      lcd_putc(' ');
      itoa( i >> 6 & 0x1F, s, 10);	// Device address
      uart_puts( s );
      uart_putc(' ');
	  lcd_puts( s );
      lcd_putc(' ');
      itoa((i & 0x3F) | (~i >> 7 & 0x40), s, 10); // Key Code
      uart_puts( s );
      uart_puts( "\n\r" );
	  lcd_puts( s );
    }
  }
}
